/********************************************************
s-rcon-win32	half-life remote console utility
Copyright (C) 2000  M. Sean Finney <seanius@yahoo.com>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

********************************************************/
#include <stdlib.h> 
#include <stdio.h> 
#include <ctype.h>
#include <string.h>  
#include "uglyUDPstuff.h"

char* preformat_cmd(char* cmd);
char* rcon_format(char* password, char* command);
int interactive_mode();
void dumb_user(char* prg);

int main(int argc, char **argv) { 
	char *host, *port, *pw, *cmd;
	int error_status=0;

	printf("rcon v1.0 (win32) -- remote console for half-life and mods\n");
	printf("questions, praise, flames, bugs, and generous donations can\n");
	printf("be sent to seanius@yahoo.com\n");

	//weak cmdline check
	if(argc!=5 && argc != 1){printf("wrong number of command line args\n"); 
		dumb_user(argv[0]); exit(-1);}
	
	//interactive mode in separate function
	if(argc==1) error_status = interactive_mode();
	else{ host=argv[1]; port=argv[2]; pw=argv[3]; cmd=preformat_cmd(argv[4]); 
		error_status = make_connection(host, port);
		if(error_status != -1)send_command((char*)rcon_format(pw, cmd));
	}
	
	WSACleanup();
	return error_status;
 } 

char* preformat_cmd(char* cmd){
	unsigned int i,whitespace_count=0, pos=0;
	char *quoted_cmd;
	for(i=0;i<strlen(cmd);i++){	if(iswspace(cmd[i])) whitespace_count++; }
	quoted_cmd=(char*)malloc(sizeof(char)*(strlen(cmd)+(whitespace_count*2)));
	for(i=0;i<strlen(cmd);i++){
		if(!iswspace(cmd[i])) quoted_cmd[pos++] = cmd[i];
		else{ quoted_cmd[pos++] = '\"'; quoted_cmd[pos++]= ' '; quoted_cmd[pos++] = '\"'; }
	}
	quoted_cmd[pos]='\0';
	return quoted_cmd;
}

char* rcon_format(char* password, char* command){
	int command_len, password_len, total_len;
	char* fcmd_final="";
	char* fcmd_prefix="\377\377\377\377rcon \"";
	char* fcmd_middle="\" \"";
	char* fcmd_end="\"";
	command_len=strlen(command);
	password_len=strlen(password);
	total_len=(strlen(fcmd_prefix)+strlen(fcmd_middle)+strlen(fcmd_end)+
        command_len+password_len);
	fcmd_final = (char *)malloc(sizeof(char)*total_len);
	sprintf(fcmd_final, "%s%s%s%s%s", fcmd_prefix, password, fcmd_middle,
        command, fcmd_end);
	return fcmd_final;
}

int interactive_mode(){
	char *host, *port, *pw, *cmd;
	int maxsize=128,num_blanklines=0,error_status=0;
	host=(char*)malloc(sizeof(char)*maxsize);
	port=(char*)malloc(sizeof(char)*maxsize);
	pw=(char*)malloc(sizeof(char)*maxsize);
	cmd=(char*)malloc(sizeof(char)*maxsize);
	
	printf("host: ");
	fgets(host, maxsize, stdin);
	//trim the newline
	host[(strlen(host)-1)]='\0';
	printf("port: ");
	fgets(port, maxsize, stdin);
	port[(strlen(port)-1)]='\0';
	printf("password: ");
	fgets(pw, maxsize, stdin);
	pw[(strlen(pw)-1)]='\0';

	error_status=make_connection(host, port);
	if(error_status == -1) return -1;

	printf("the console is yours. (two blank lines to quit)\n");
	while(num_blanklines<2){
		fgets(cmd, maxsize, stdin);
		if(cmd[0]=='\n') num_blanklines++;
		else { cmd[(strlen(cmd)-1)]='\0';
		send_command(rcon_format(pw, preformat_cmd(cmd))); }
	}
	return error_status;}

void dumb_user(char* prg){
	printf("usage:\n");
	printf("rcon hostname port password \"command\"\n");
	printf("or just plain ol' rcon (interactive mode)\n");
}
